from flask import Blueprint, request, jsonify, make_response
from sqlalchemy import desc, and_
from app.rooms.models import Room
from app.location_log.models import LocationLogHistory
from app import db
from app.utils import token_required, paginated_query
from datetime import datetime, timedelta
from app.location_log.utills import append_location_time_line_array, save_location_log_history
import pytz
import json
from app.logs.utils import stringify_dates
from app.users.models import User
from app.patients.models import Patient

locationlogs = Blueprint('location_logs', __name__)

def add_location_log(device_id, dt_mac, site_id, device_name, room_id, device_site_id):
    try:
        utc_now = datetime.now(pytz.utc)
        la_timezone = pytz.timezone('America/Los_Angeles')
        la_time = utc_now.astimezone(la_timezone)
        formatted_time = la_time.strftime('%Y-%m-%d %H:%M:%S')
        existing_log = LocationLogHistory.query.filter(and_(LocationLogHistory.device_id==device_id, LocationLogHistory.site_id== device_site_id)).order_by(desc(LocationLogHistory.
                                                                                             created_at)).first()
        check_device_patient = Patient.query.filter_by(device_id=device_id).first()
        check_device_user = User.query.filter_by(device_id=device_id).first()
        room = Room.query.filter_by(id=room_id).first()
        if check_device_patient:
            flag = 0
            user_or_patient_name = check_device_patient.patient_name
        elif check_device_user:
            flag = 1
            user_name = json.loads(check_device_user.full_name)
            user_or_patient_name = f"{user_name['first_name']} {user_name['last_name']}"
        else:
            raise ValueError("Device is not associated with any patient or user.")
        if existing_log:
            after_24hrs_time = existing_log.created_at + timedelta(hours=24)
            if formatted_time >= str(after_24hrs_time) :
                save_location_log_history(device_id, device_name, dt_mac, user_or_patient_name, room,
                                          device_site_id, formatted_time, flag)
            elif existing_log.room_id != room_id:
                existing_log.room_id = room_id
                append_location_time_line_array(existing_log, room.room_name, user_or_patient_name, formatted_time)
        elif site_id == device_site_id:
            save_location_log_history(device_id, device_name, dt_mac, user_or_patient_name, room,
                                      device_site_id, formatted_time, flag)
        db.session.commit()
    except Exception as e:
        db.session.rollback()
        raise ValueError(f"Error in adding location log: {str(e)}")
@locationlogs.route("/api/v1/get/location_logs", methods=['GET'])
@token_required(1, 2, 3)
def get_location_logs(current_user):
    try:
        from_date = request.args.get('fromDate')
        to_date = request.args.get('toDate')
        staff_patient = request.args.get('logType')
        logs_query = LocationLogHistory.query.filter_by(flag=1 if staff_patient == "staff" else 0).order_by(
            LocationLogHistory.updated_at.desc())
        if from_date and to_date:
            dates = stringify_dates(from_date, to_date)
            logs_query = logs_query.filter(LocationLogHistory.created_at.between(dates[0], dates[1]))
        location_logs, data = paginated_query(logs_query,
            [LocationLogHistory.location_log_arr],
                        LocationLogHistory.site_id)
        location_log_list = []
        for location in location_logs:
            location_data = json.loads(location.location_log_arr)
            time_line_log = json.loads(location.location_timeline_arr)
            formatted_time_line_log = []
            for time_line in time_line_log:
                start_time = datetime.fromisoformat(time_line['start_time'])
                end_time = datetime.fromisoformat(time_line['end_time']) if time_line['end_time'] else None
                start_time_str = start_time.astimezone(pytz.utc).strftime('%Y-%m-%d %H:%M:%S')
                end_time_str = end_time.astimezone(pytz.utc).strftime('%Y-%m-%d %H:%M:%S') if end_time else ""
                formatted_time_line_log.append({
                    "room_name": time_line['room_name'],
                    "name": time_line['name'],
                    "start_time": start_time_str,
                    "end_time": end_time_str,
                    "duration": time_line['duration']})
            location_log_list.append({
                "id": location.id,
                "device_name": location_data['device_name'],
                "device_mac": location_data['dt_mac'],
                "room_id": location.room_id,
                "name": location_data['user_patient_name'],
                "room_name": location_data['room_name'],
                "time_line_log": formatted_time_line_log,
                "created_at": location.created_at.strftime('%d %b %Y %I:%M:%S %p'),
                "updated_at": location.updated_at})
        return jsonify({'location': location_log_list, 'data': data}), 200
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', "status": "error"}), 500)